/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2008, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.maven.plugins.as.control.mojo;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.HashSet;
import java.util.Set;
import java.util.StringTokenizer;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jboss.maven.plugins.as.common.JbossAsRuntimeDelegate;

/**
 * JbossAsControlAbstractDeploymentMojo.java
 * 
 * Base class for Maven2 deployment mojos
 * 
 * @author <a href="mailto:andrew.rubinger@jboss.org">ALR</a>
 * @version $Revision: $
 * 
 */
public abstract class JbossAsControlAbstractDeploymentMojo extends JbossAsControlAbstractMojo
{
   // Class Members

   protected static final Log logger = LogFactory.getLog(JbossAsControlAbstractDeploymentMojo.class);

   /**
    * Delimter separating out paths of configured files to deploy or undeploy
    */
   protected static final String DELIMITER_DEPLOYABLE_FILES = ",";

   // Maven Configuration Properties

   /**
    * Whether or not to run the integration tests
    * 
    * @parameter alias="jboss.test.run" default-value=false expression="${jboss.test.run}"
    */
   protected boolean runTests;

   /**
    * Server configuration to use
    * 
    * @parameter expression="${jboss.serverConfigName}"
    * @required
    */
   protected String serverConfigName;

   /**
    * Comma-delimited list of files to deploy/undeploy
    * 
    * @parameter
    * @required
    */
   protected String files;

   /**
    * Delegate for JBossAS Runtime Operations
    */
   protected final JbossAsRuntimeDelegate delegate = new JbossAsRuntimeDelegate();
   
   /*
    * The classloader with jbossall-client.jar added.
    * */
   protected static ClassLoader hackClassLoader;

   /**
    * Obtains a Set of Deployable Units to deploy
    * 
    * @return
    */
   protected Set<File> getDeployableUnits()
   {
      // Initialize
      Set<File> deployables = new HashSet<File>();

      // Get the configured files, comma-delimited
      String configuredFiles = this.getFiles();
      if (configuredFiles != null)
      {
         configuredFiles = configuredFiles.trim();
      }

      // Ensure specified
      if (configuredFiles == null || configuredFiles.length() == 0)
      {
         throw new RuntimeException("Configured files must be specified for deployment");
      }

      // Get a tokenizer to split up the possible list
      StringTokenizer tokenizer = new StringTokenizer(configuredFiles,
            DELIMITER_DEPLOYABLE_FILES);

      // Add each deployable to the Set
      while (tokenizer.hasMoreTokens())
      {
         // Get the configured deployable file
         String deployable = tokenizer.nextToken();
         File file = new File(deployable);

         // Ensure File exists
         if (!file.exists())
         {
            throw new RuntimeException("Configured file for deployment, " + deployable + ", does not exist.");
         }

         // Add to the Set
         deployables.add(file);
      }

      // Return
      return deployables;
   }
   
   /*
   * Get a ClassLoader granting visibility to classes required
   * for JMX Invocation.  Ideally these should be declared as proper dependencies of
   * server-manager, but the classes cannot be bound to a particular JBossAS version.
   * So we go to the user's library.
   */
   protected ClassLoader getJbossClientClassLoader(ClassLoader oldLoader)
   {

      if(hackClassLoader == null)
      {
          //TODO Hack
          // Construct a path to jbossall-client using $JBOSS_HOME
          String jbossClientPathName = this.delegate.getJbossHome() + "/client/jbossall-client.jar";
          File jbossClientPath = new File(jbossClientPathName);
          if (!jbossClientPath.exists() || jbossClientPath.isDirectory())
          {
             throw new RuntimeException("Could not find JBossAS Client JAR located at " + jbossClientPathName
                   + ", required for deploy.");
          }
          URL jbossClientURL = null;
          try
          {
             jbossClientURL = jbossClientPath.toURL();
          }
          catch (MalformedURLException e)
          {
             throw new RuntimeException(e);
          }

          // Create a new CL pointing to jbossall-client, using a parent of the old CL
          hackClassLoader = new URLClassLoader(new URL[]
          {jbossClientURL}, oldLoader);
      }
      
      return hackClassLoader;      
   }

   // Accessors / Mutators

   public boolean isEnabled()
   {
      // If this Mojo is configured, it runs
      return this.runTests;
   }

   public String getServerConfigName()
   {
      return serverConfigName;
   }

   public void setServerConfigName(String serverConfigName)
   {
      this.serverConfigName = serverConfigName;
   }

   public String getFiles()
   {
      return files;
   }

   public void setFiles(String files)
   {
      this.files = files;
   }
}
