/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2008, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.maven.plugins.as.control.mojo;

import java.io.IOException;

import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;
import org.jboss.maven.plugins.as.common.JbossAsConfigurationDelegate;

/**
 * JbossAsControlCreateConfigurationMojo
 * 
 * Maven2 Plugin to create a JBossAS Server Configuration
 * 
 * @author <a href="mailto:andrew.rubinger@jboss.org">ALR</a>
 * @version $Revision: $
 * @phase pre-integration-test
 * @goal create-config
 */
public class JbossAsControlCreateConfigurationMojo extends JbossAsControlAbstractMojo
{

   /**
    * Whether or not to run the integration tests
    * 
    * @parameter alias="jboss.createConfig.skip" default-value=false expression="${jboss.createConfig.skip}"
    */
   private boolean skipConfig;

   /**
    * Server configuration to use as base
    * 
    * @parameter
    * @required
    */
   private String serverConfigBase;

   /**
    * Server configuration name to create
    * 
    * @parameter
    * @required
    */
   private String serverConfigName;

   /**
    * Location of the overlay files
    * 
    * @parameter alias='jboss.overlayLocation' default-value='${basedir}/src/main/resources'
    */
   private String overlayLocation;

   /**
    * Whether or not to overwrite an existing configuration
    * 
    * @parameter default-value=false
    */
   private boolean overwrite;

   /**
    * Delegate for configuration business logic
    */
   private final JbossAsConfigurationDelegate delegate = new JbossAsConfigurationDelegate();

   // Functional Methods

   /**
    * Maven Plugin Execution Entry
    * 
    * Stops the Server, assuming only one server instance is running and failing if not.
    */
   public void execute() throws MojoExecutionException, MojoFailureException
   {
      // If we're skipping the tests, return
      if (!this.isEnabled())
      {
         this.getLog().info("Skipping configuration creation, to enable set \"jboss.createConfig.skip\" to false.");
         return;
      }

      // Create the new server configuration
      try
      {
         delegate.createServerConfiguration(delegate.getJbossHome(), this.getServerConfigName(), this
               .getServerConfigBase(), this.getOverlayLocation(), this.isOverwrite());
      }
      catch (IOException e)
      {
         throw new MojoExecutionException(e.getMessage(), e);
      }
   }

   // Accessors / Mutators

   public boolean isEnabled()
   {
      return !this.skipConfig;
   }

   public String getServerConfigBase()
   {
      return serverConfigBase;
   }

   public void setServerConfigBase(String serverConfigBase)
   {
      this.serverConfigBase = serverConfigBase;
   }

   public String getServerConfigName()
   {
      return serverConfigName;
   }

   public void setServerConfigName(String serverConfigName)
   {
      this.serverConfigName = serverConfigName;
   }

   public String getOverlayLocation()
   {
      return overlayLocation;
   }

   public void setOverlayLocation(String overlayLocation)
   {
      this.overlayLocation = overlayLocation;
   }

   public boolean isOverwrite()
   {
      return overwrite;
   }

   public void setOverwrite(boolean overwrite)
   {
      this.overwrite = overwrite;
   }

}
